<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class QueryFareMasterPricerTravelBoardSearchAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected string $searchCriteriaString;
    protected string $passengerString;
    protected int $numberOfPassengers;
    protected array $searchCriteriaFlight;
    protected array $cabinPreference;
    protected ?array $sources;
    protected ?array $carriers;
    protected ?array $flightType;
    protected bool $isFlexibleDates;

    public function execute(
        array  $passengerCriteria,
        array  $searchCriteriaFlight,
        ?array $searchModifiers,
    ): array {
        try {
            $this->searchCriteriaFlight = $searchCriteriaFlight;
            //            $this->searchCriteriaString = $this->generateSearchCriteria($searchCriteriaFlight);
            $this->passengerString = $this->generatePassengerCriteria($passengerCriteria);
            $this->cabinPreference = $searchModifiers['CabinPreference'];
            $this->sources = $searchModifiers['Source'] ?? ["GDS", "NDC"];
            $this->carriers = $searchModifiers['Carriers'] ?? [];
            $this->flightType = $searchModifiers["flightType"];
            $this->isFlexibleDates = $searchModifiers["isFlexibleDates"];


            $response = $this->createCurlRequest();

            self::storeInFile("xml", "0-flight-search-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json",
                "0-flight-search-response",
                json_encode($responseArray)
            );


            return $responseArray;
        } catch (\Exception $exception) {
            $e = "Error in QueryFareMasterPricerTravelBoardSearchAction:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xml = $this->xmlHeader() . '
          <soap:Body>
            <Fare_MasterPricerTravelBoardSearch>'

            . $this->createNumberOfUnitsString() .

            $this->passengerString .



            $this->createFareFamiliesString() .

            $this->createFareOptionsString() .



            $this->createTravelFlightInfoString() .

            $this->generateSearchCriteria() .

            '</Fare_MasterPricerTravelBoardSearch>
          </soap:Body>

        </soap:Envelope>';

        self::storeInFile("xml", "0-flight-search-request", $xml);

        return $xml;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/FMPTBQ_23_2_1A";
    }

    protected function generateSearchCriteria(): string
    {
        $xmlSearchNode = "";

        foreach ($this->searchCriteriaFlight as $index => $criteria) {
            list($from, $to, $departure) = $this->getSearchCriteria($criteria);

            $refNo = $index + 1;

            $xmlSearchNode .= "
                    <itinerary>
                        <requestedSegmentRef>
                          <segRef>$refNo</segRef>
                        </requestedSegmentRef>
                        <departureLocalization>
                          <departurePoint>
                            <locationId>$from</locationId>
                          </departurePoint>
                        </departureLocalization>
                        <arrivalLocalization>
                          <arrivalPointDetails>
                            <locationId>$to</locationId>
                          </arrivalPointDetails>
                        </arrivalLocalization>
                        <timeDetails>
                          <firstDateTimeDetail>
                            <date>$departure</date>
                          </firstDateTimeDetail>
                        " . $this->getRangeOfDateString() . "
                        </timeDetails>
                    </itinerary>
            ";
        }


        return $xmlSearchNode;
    }

    protected function getRangeOfDateString(): string
    {
        if ($this->isFlexibleDates === false) {
            return "";
        }

        return "
            <rangeOfDate>
                <rangeQualifier>C</rangeQualifier>
                <dayInterval>1</dayInterval>
            </rangeOfDate>
        ";
    }

    protected function getSearchCriteria(array $searchCriteriaFlight): array
    {
        $from = $searchCriteriaFlight['From']['value'];
        $to = $searchCriteriaFlight['To']['value'];
        $departureCarbonDate = Carbon::parse($searchCriteriaFlight['departureDate']);
        $departure = $departureCarbonDate->format("d") . $departureCarbonDate->format("m") . $departureCarbonDate->format("y");
        return array($from, $to, $departure);
    }

    protected function generatePassengerCriteria(array $passengerCriteria): string
    {
        $adt = 0;
        $chd = 0;
        $inf = 0;

        foreach ($passengerCriteria as $criterion) {
            if ($criterion['passengerTypeCode'] === "ADT") {
                $adt += intval($criterion['number']);
            } elseif ($criterion['passengerTypeCode'] === "CHD") {
                $chd += intval($criterion['number']);
            } else {
                $inf += intval($criterion['number']);
            }
        }

        $this->numberOfPassengers = $adt + $chd;

        $criteriaString = $this->createPTCWiseString($adt, "ADT", 1);

        if ($chd > 0) {
            $criteriaString .= $this->createPTCWiseString($chd, "CHD", $adt + 1);
        }

        if ($inf > 0) {
            $criteriaString .= $this->createInfantString($inf);
        }

        return $criteriaString;
    }

    protected function createPTCWiseString(int $passengerCount, string $ptc, int $startNumber): string
    {
        $s = "<paxReference><ptc>$ptc</ptc>";

        for ($x = $startNumber; $x < $passengerCount + $startNumber; $x++) {
            $s .= "<traveller><ref>$x</ref></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }

    protected function createInfantString(int $infantCount): string
    {
        $s = "<paxReference><ptc>INF</ptc>";

        for ($x = 1; $x <= $infantCount; $x++) {
            $s .= "<traveller><ref>$x</ref><infantIndicator>1</infantIndicator></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }

    protected function createFareFamiliesString(): string
    {
        $xml = "";

        foreach ($this->cabinPreference['cabins'] as $index => $cabin) {
            $refNumber = $index + 1;

            $xml .= "
            <fareFamilies>
                <familyInformation>
                  <refNumber>$refNumber</refNumber>
                  <fareFamilyname>" . $this->getCabinID($cabin)['family'] . "</fareFamilyname>
                  <hierarchy>40</hierarchy>
                </familyInformation>
                <familyCriteria>
                  <cabinProduct>
                    <cabinDesignator>" . $this->getCabinID($cabin)['designator'] . "</cabinDesignator>
                  </cabinProduct>

                </familyCriteria>
            </fareFamilies>
            ";
            //                  <cabinProcessingIdentifier>MC</cabinProcessingIdentifier>
        }

        return $xml;
    }

    protected function getCabinID(string $key)
    {
        $cabins = new Collection([
            ["name" => "Economy", "designator" => "M", "family" => "EcoY"],
            ["name" => "PremiumEconomy", "designator" => "W", "family" => "EcoW"],
            ["name" => "Business", "designator" => "C", "family" => "Business"],
            ["name" => "First", "designator" => "F", "family" => "First"],
            ["name" => "PremiumFirst", "designator" => "F", "family" => "First"],
        ]);


        return $cabins->firstWhere('name', $key);
    }

    protected function createNumberOfUnitsString(): string
    {
        $xml = "
            <numberOfUnit>
                <unitNumberDetail>
                  <numberOfUnits>" . $this->numberOfPassengers . "</numberOfUnits>
                  <typeOfUnit>PX</typeOfUnit>
                </unitNumberDetail>";

        if (in_array("GDS", $this->sources)) {
            $xml .= "
                <unitNumberDetail>
                  <numberOfUnits>" . $this->getNumberOfUnitsForFareMasterPricer() . "</numberOfUnits>
                  <typeOfUnit>RC</typeOfUnit>
                </unitNumberDetail>
            ";
        }

        if (in_array("NDC", $this->sources)) {
            $xml .= "
                <unitNumberDetail>
                    <numberOfUnits>" . $this->getNumberOfUnitsNDC() . "</numberOfUnits>
                    <typeOfUnit>NDC</typeOfUnit>
                </unitNumberDetail>
            ";
        }

        $xml .= "</numberOfUnit>";

        return $xml;
    }

    protected function createFareOptionsString(): string
    {
        $xml = "
            <fareOptions>
                <pricingTickInfo>
                    <pricingTicketing>

        ";

        if (!in_array("NDC", $this->sources)) {
            $xml .= "<priceType>XND</priceType>";
        }

        $xml .= "
                        <priceType>ET</priceType>
                        <priceType>TAC</priceType>
                        <priceType>RP</priceType>
                        <priceType>IFS</priceType>
                        <priceType>RU</priceType>
                    </pricingTicketing>
                </pricingTickInfo>
            </fareOptions>
        ";

        return $xml;
    }

    protected function createTravelFlightInfoString(): string
    {
        if (count($this->carriers) === 0) {
            return "";
        }

        $xml = "
            <travelFlightInfo>
        ";

        foreach ($this->carriers as $carrier) {
            $xml .= "
                <companyIdentity>
                    <carrierQualifier>M</carrierQualifier>
                    <carrierId>$carrier</carrierId>
                </companyIdentity>
            ";
        }

        $xml .= "
            </travelFlightInfo>
        ";

        return $xml;
    }
}
